#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Create web page favicon.png.

use 5.006;
use strict;
use warnings;
use File::chdir;
use File::stat;
use File::Slurp;
use File::Temp;
use Image::Xpm;
use IPC::Run;
use List::Util 'max','min';
use Image::ExifTool;

use FindBin;
use lib $FindBin::Bin;


my $level = 8;
my $chamfer = .2;
my $AlphaBits = 4;   # 1,2,4

my $version = 0;
foreach my $arg (@ARGV) {
  if ($arg =~ /^VERSION=(.*)/) {
    $version = $1;
  }
}

my $program_filename = File::Spec->catfile($FindBin::Bin,
                                           $FindBin::Script);
my $png_filename = File::Spec->catfile($FindBin::Bin,
                                       'favicon.png');

if (1) {
  my $image = Image::Xpm->new(-width => 16, -height => 16);
  $image->rectangle(0,0,15,15,'white',1);
  my $w = 5;
  my $grey = '#'.('B8'x3);
  # horizontal
  foreach my $i (0 .. 4) {
    my $x = $i;
    my $y = $w + $i;
    $image->line($x,$y, $x+2*$w-1,$y, $grey);
  }
  # vertical
  foreach my $i (0 .. 4) {
    my $x = 3*$w-1 - $i;
    my $y = $i;
    $image->line($x,$y, $x,$y+2*$w, $grey);
  }
  # triangle overlap
  foreach my $i (0 .. 4) {
    my $x = 2*$w;
    my $y = $w + $i;
    $image->line($x,$y, $x+$i,$y, 'black');
  }
  $image->save('favicon.xpm');
  # IPC::Run::run(['cat','favicon.xpm']);
  IPC::Run::run(['convert','favicon.xpm','favicon.png']);
}

if (0) {
  my $temp = File::Temp->newdir;
  my $tempdir = $temp->dirname;
  local $CWD = $tempdir;

  open my $tex_fh, '>', 'favicon.tex' or die;

  print $tex_fh <<"HERE";
\\documentclass[a4paper]{article}
\\pagestyle{empty}
\\usepackage{tikz}
\\begin{document}
\\begin{tikzpicture}[scale=.117]
  \\begin{scope}[black!20]
  \\fill (0,0)--(1,0)--(1,1)--(-1,1)--cycle;
  \\fill (1,0)--(1,-1)--(2,0)--cycle;
  \\fill (2,0)--(2,2)--(1,1)--cycle;
  \\end{scope}
  \\fill[black] (1,0)--(2,0)--(1,1)--cycle;
\\end{tikzpicture}
\\end{document}
HERE
  close $tex_fh or die;
  IPC::Run::run(['cat','favicon.tex']);

  IPC::Run::run(['latexmk','-pdf','favicon.tex'], '<',File::Spec->devnull);

  IPC::Run::run(['pdfcrop',
                 '--verbose',
                 '--margins','.1',
                 'favicon.pdf',
                 'favicon-cropped.pdf'],
                '<', File::Spec->devnull)
      or die;
  rename 'favicon-cropped.pdf', 'favicon.pdf' or die;

  print "gs\n";
  IPC::Run::run(['gs',
                 '-q', # quiet, cf -dQUIET
                 '-dNOPAUSE',
                 '-r100',
                 '-sDEVICE=pnggray',
                 "-dTextAlphaBits=$AlphaBits",     # grey scale edge blurring
                 "-dGraphicsAlphaBits=$AlphaBits",
                 "-sOutputFile=$png_filename",
                 'favicon.pdf'],
                '<', File::Spec->devnull)
      or die;
  print "\n";
}


# PNG spec 11.3.4.2 suggests RFC822 (or rather RFC1123) for CreationTime
use constant STRFTIME_FORMAT_RFC822 => '%a, %d %b %Y %H:%M:%S %z';
my $stat = File::stat::stat($program_filename)
  or die "Oops, cannot stat $program_filename: $!";
my $creation_time = POSIX::strftime(STRFTIME_FORMAT_RFC822,
                                    localtime($stat->mtime));

my $copyright = File::Slurp::read_file($program_filename);
$copyright =~ /^# (Copyright.*)/m or die;
$copyright = $1;

pngtextadd($png_filename, 'Author',    'Kevin Ryde');
pngtextadd($png_filename, 'Generator', "tools/$FindBin::Script");
pngtextadd($png_filename, 'Title',     'Iterations of the Dragon Curve - sample favicon');
pngtextadd($png_filename, 'Creation Time', $creation_time);
pngtextadd($png_filename, 'Copyright', $copyright);
pngtextadd($png_filename, 'Homepage',  'http://user42.tuxfamily.org/seq-A073057-append-morphism/index.html');
# pngtextadd($png_filename, 'Version',   $version);

print "uncompressed ",-s $png_filename,"\n";
IPC::Run::run(['optipng','-quiet','-o5',$png_filename]);
print "compressed   ",-s $png_filename,"\n";


# IPC::Run::run(['xpdf','favicon.pdf']);
# IPC::Run::run(['xzgv',$png_filename]);
# IPC::Run::run(['pnginfo',$png_filename]);

{
  my $exifTool = Image::ExifTool->new;
  my $href = $exifTool->ImageInfo($png_filename);
  my $width = $href->{'ImageWidth'};
  my $height = $href->{'ImageHeight'};
  print "size $width x $height\n";
  ($width == 16 && $height == 16) or die;
}

exit 0;

# add text to the png favicon in $png_filename
sub pngtextadd {
  my ($filename, $keyword, $value) = @_;
  system('pngtextadd', "--keyword=$keyword", "--text=$value", $filename) == 0
    or die "system(pngtextadd)";
}
